/*
 *   (C) Copyright IBM Corp. 2006
 *   (C) Copyright ALT Linux Ltd. 2011
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Module: BtrFS FSIM
 * File: evms2/engine/plugins/btrfs/fsimbtrfs.h
 */

extern plugin_record_t		btrfs_plugrec;
extern plugin_record_t		*my_plugin_record;
extern engine_functions_t	*EngFncs;

#define MKFS_LABEL_INDEX	0
#define MKFS_LABEL_NAME		"label"
#define MKFS_OPTIONS_COUNT	1

// XXX: #define MAX_LABEL_LEN	256
#define MAX_LABEL_LEN		60

#define BTRFS_MIN_SIZE		(256 * 1024 * 1024)
#define BTRFS_MAGIC		"_BHRfS_M"

// XXX: #define BTRFS_SUPER_LOC	0x4000
#define BTRFS_SUPER_LOC		0x10000

#define BTRFS_CSUM_SIZE		32
#define BTRFS_FSID_SIZE		16

struct btrfs_super_block
{
	u_int8_t	csum[BTRFS_CSUM_SIZE];
	u_int8_t	fsid[BTRFS_FSID_SIZE];
	u_int64_t	bytenr;
	u_int64_t	flags;
	u_int64_t	magic;
	u_int64_t	generation;
	u_int64_t	root;
	u_int64_t	chunk_root;
	u_int64_t	log_root;
	u_int64_t	log_root_transid;
	u_int64_t	total_bytes;
	u_int64_t	bytes_used;
	u_int64_t	root_dir_objectid;
	u_int64_t	num_devices;
	u_int32_t	sectorsize;
	u_int32_t	nodesize;
	u_int32_t	leafsize;
	u_int32_t	stripesize;
	u_int32_t	sys_chunk_array_size;
	u_int64_t	chunk_root_generation;
	u_int64_t	compat_flags;
	u_int64_t	compat_ro_flags;
	u_int64_t	incompat_flags;
	u_int16_t	csum_type;
	u_int8_t	root_level;
	u_int8_t	chunk_root_level;
	u_int8_t	log_root_level;
} __attribute__ ((__packed__));

#define BTRFS_SIZE_OF_SUPER_BLOCK	sizeof(struct btrfs_super_block)

#define EVMS_IS_MOUNTED(vol)		((vol)->mount_point != 0)
#define EVMS_GET_DEVNAME(vol)		((vol)->name)

#define BTRFS_MKFS_PROG			"mkfs.btrfs"
