/*
 *   (C) Copyright IBM Corp. 2006
 *   (C) Copyright ALT Linux Ltd. 2012
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Module: LUKS
 * File: evms2/engine/plugins/luks/luks.h
 */

extern plugin_record_t		*my_plugin_record;
extern engine_functions_t	*EngFncs;

#define MKFS_OPTIONS_COUNT	0

#define LUKS_CIPHERNAME_L	32
#define LUKS_CIPHERMODE_L	32
#define LUKS_HASHSPEC_L		32
#define LUKS_DIGESTSIZE		20
#define LUKS_HMACSIZE		32
#define LUKS_SALTSIZE		32
#define LUKS_NUMKEYS		8
#define LUKS_UUID_STRING_L	40

#define LUKS_MAGIC		"LUKS\xba\xbe"
#define LUKS_MAGIC_SIZE		6

#define LUKS_METADATA_OFFSET	0

typedef struct luks_metadata_s
{
	char		magic[LUKS_MAGIC_SIZE];
	u_int16_t	version;
	char		cipherName[LUKS_CIPHERNAME_L];
	char		cipherMode[LUKS_CIPHERMODE_L];
	char		hashSpec[LUKS_HASHSPEC_L];
	u_int32_t	payloadOffset;
	u_int32_t	keyBytes;
	char		mkDigest[LUKS_DIGESTSIZE];
	char		mkDigestSalt[LUKS_SALTSIZE];
	u_int32_t	mkDigestIterations;
	char		uuid[LUKS_UUID_STRING_L];

	struct
	{
		u_int32_t	active;
		u_int32_t	passwordIterations;
		char		passwordSalt[LUKS_SALTSIZE];
		u_int32_t	keyMaterialOffset;
		u_int32_t	stripes;
        } keyblock[LUKS_NUMKEYS];

	char		_padding[432];
} __attribute__ ((__packed__)) luks_metadata_t;

#define LUKS_SIZE_METADATA		sizeof(luks_metadata_t)
#define LUKS_METADATA_SECTOR_COUNT	(sizeof(luks_metadata_t)/EVMS_VSECTOR_SIZE)

typedef struct guid_s
{
	u_int32_t	time_low;
	u_int16_t	time_mid;
	u_int16_t	time_high;
	u_int8_t	clock_seq_high;
	u_int8_t	clock_seq_low;
	u_int8_t	node[6];
} __attribute__((packed)) guid_t;

typedef struct luks_private_data_s
{
	char		cipher[EVMS_NAME_SIZE + 1];
	guid_t		uuid;
	char		password[EVMS_NAME_SIZE + 1];
} luks_private_data_t;

#define LUKS_PLUGIN_INFO_COUNT				6

#define LUKS_PLUGIN_INFO_SNAME_INDEX			0
#define LUKS_PLUGIN_INFO_LNAME_INDEX			1
#define LUKS_PLUGIN_INFO_TYPE_INDEX			2
#define LUKS_PLUGIN_INFO_VERSION_INDEX			3
#define LUKS_PLUGIN_INFO_REQENGSVCVERSION_INDEX		4
#define LUKS_PLUGIN_INFO_REQPLUGAPIVERSION_INDEX	5

#define LUKS_CIPHER_DEFAULT		"aes-cbc-essiv:sha256"

#define LUKS_CREATE_OPTION_COUNT	0

/*	No need anymore
 *
#define LUKS_CREATE_CIPHER_INDEX	0
#define LUKS_CREATE_CIPHER_NAME		"Cipher"
#define LUKS_CREATE_PASSWORD_INDEX	1
#define LUKS_CREATE_PASSWORD_NAME	"Password"
*/

#define LUKS_INFO_COUNT			3

#define LUKS_INFO_NAME_INDEX		0
#define LUKS_INFO_SIZE_INDEX		1
#define LUKS_INFO_CIPHER_INDEX		2
