/*
 *   (C) Copyright IBM Corp. 2006
 *   (C) Copyright ALT Linux Ltd. 2012
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Module: LUKS
 * File: evms2/engine/plugins/luks/utils.c
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <plugin.h>
#include <libcryptsetup.h>

#include "luks.h"
#include "utils.h"

int utils_get_luks_list(list_anchor_t *list)
{
	int rc;
	list_anchor_t tlist;

	LOG_ENTRY();

	tlist = EngFncs->allocate_list();

	REQUIRE(tlist != NULL);

	rc = EngFncs->get_object_list(0, DATA_TYPE, my_plugin_record, NULL, 0, &tlist);
	if (rc == 0) *list = tlist;
	else EngFncs->destroy_list(tlist);

	LOG_EXIT_INT(rc);
	return rc;
};

static int initialize_create_option_descriptors(task_context_t *context)
{
	int rc = 0;

	LOG_ENTRY();

/*	No need anymore
 *
	REQUIRE(context != NULL);

	if (context->option_descriptors->count == LUKS_CREATE_OPTION_COUNT)
	{
		strncpy(context->option_descriptors->option[LUKS_CREATE_CIPHER_INDEX].value.s, LUKS_CIPHER_DEFAULT, EVMS_NAME_SIZE);
	}
	else
	{
		LOG_ERROR("error, wrong number of option descriptors ... count= %d\n", context->option_descriptors->count);
		rc = EINVAL;
	}
*/

	LOG_EXIT_INT(rc);
	return rc;
};

int utils_set_create_object(task_context_t *context, list_anchor_t declined_objects, task_effect_t *effect)
{
	int rc;

	LOG_ENTRY();

	REQUIRE(context != NULL);
	REQUIRE(declined_objects != NULL);
	REQUIRE(effect != NULL);
	REQUIRE(EngFncs->list_count(context->selected_objects) == 1);

	rc = initialize_create_option_descriptors(context);

	if (rc == 0) *effect |= EVMS_Effect_Reload_Options;
	else rc = EINVAL;

	LOG_EXIT_INT(rc);
	return rc;
};

int utils_allocate_create_option_descriptors(task_context_t *context)
{
	int rc = 0;

	LOG_ENTRY();

	REQUIRE(context != NULL);

	context->option_descriptors->count = LUKS_CREATE_OPTION_COUNT;

/*	No need anymore
 *
	context->option_descriptors->option[LUKS_CREATE_CIPHER_INDEX].flags		= EVMS_OPTION_FLAGS_NOT_REQUIRED;
	context->option_descriptors->option[LUKS_CREATE_CIPHER_INDEX].name		= EngFncs->engine_strdup(LUKS_CREATE_CIPHER_NAME);
	context->option_descriptors->option[LUKS_CREATE_CIPHER_INDEX].min_len		= 0;
	context->option_descriptors->option[LUKS_CREATE_CIPHER_INDEX].max_len		= EVMS_NAME_SIZE;
	context->option_descriptors->option[LUKS_CREATE_CIPHER_INDEX].tip		= EngFncs->engine_strdup(_("Cipher specification string."));
	context->option_descriptors->option[LUKS_CREATE_CIPHER_INDEX].title		= EngFncs->engine_strdup(_("Cipher"));
	context->option_descriptors->option[LUKS_CREATE_CIPHER_INDEX].type		= EVMS_Type_String;
	context->option_descriptors->option[LUKS_CREATE_CIPHER_INDEX].value.s		= EngFncs->engine_alloc(EVMS_NAME_SIZE + 1);
	if (context->option_descriptors->option[LUKS_CREATE_CIPHER_INDEX].value.s) memset(context->option_descriptors->option[LUKS_CREATE_CIPHER_INDEX].value.s, 0, EVMS_NAME_SIZE);
	else
	{
		rc = ENOMEM;
		LOG_EXIT_INT(rc);
		return rc;
	}

	context->option_descriptors->option[LUKS_CREATE_PASSWORD_INDEX].flags		= EVMS_OPTION_FLAGS_NO_INITIAL_VALUE;
	context->option_descriptors->option[LUKS_CREATE_PASSWORD_INDEX].name		= EngFncs->engine_strdup(LUKS_CREATE_PASSWORD_NAME);
	context->option_descriptors->option[LUKS_CREATE_PASSWORD_INDEX].min_len		= 1;
	context->option_descriptors->option[LUKS_CREATE_PASSWORD_INDEX].max_len		= EVMS_NAME_SIZE;
	context->option_descriptors->option[LUKS_CREATE_PASSWORD_INDEX].tip		= EngFncs->engine_strdup(_("Enter password."));
	context->option_descriptors->option[LUKS_CREATE_PASSWORD_INDEX].title		= EngFncs->engine_strdup(_("Password"));
	context->option_descriptors->option[LUKS_CREATE_PASSWORD_INDEX].type		= EVMS_Type_String;
	context->option_descriptors->option[LUKS_CREATE_PASSWORD_INDEX].value.s		= EngFncs->engine_alloc(EVMS_NAME_SIZE + 1);
	if (context->option_descriptors->option[LUKS_CREATE_PASSWORD_INDEX].value.s) memset(context->option_descriptors->option[LUKS_CREATE_PASSWORD_INDEX].value.s, 0, EVMS_NAME_SIZE);
	else
	{
		rc = ENOMEM;
		LOG_EXIT_INT(rc);
		return rc;
	}
*/

	LOG_EXIT_INT(rc);
	return rc;
};

int utils_get_acceptable_create_objects(task_context_t * context)
{
	int rc = EINVAL;
	uint count;

	LOG_ENTRY();

	REQUIRE(context != NULL);

	count = EngFncs->list_count(context->acceptable_objects);

	REQUIRE(count == 0);

	rc = EngFncs->get_object_list(0, DATA_TYPE, NULL, NULL, VALID_INPUT_OBJECT, &context->acceptable_objects);

	LOG_EXIT_INT(rc);
	return rc;
};

int utils_set_create_option(task_context_t *context, u_int32_t index, value_t *value, u_int32_t *UNUSED(info))
{
	int rc = 0;

	LOG_ENTRY();

/*	No need anymore
 *
	REQUIRE(context != NULL);
	REQUIRE(value != NULL);

	if (index == LUKS_CREATE_CIPHER_INDEX)
	{
		if (value->s == NULL || value->s[0] == 0) strncpy(context->option_descriptors->option[index].value.s, LUKS_CIPHER_DEFAULT, EVMS_NAME_SIZE);
		else if (strlen(value->s) > EVMS_NAME_SIZE) rc = EINVAL;
		else strcpy(context->option_descriptors->option[index].value.s, value->s);
	}
	else if (index == LUKS_CREATE_PASSWORD_INDEX)
	{
		if (value->s == NULL || value->s[0] == 0 || strlen(value->s) > EVMS_NAME_SIZE) rc = EINVAL;
		else strcpy(context->option_descriptors->option[index].value.s, value->s);
	}
	else rc = EINVAL;
*/

	LOG_EXIT_INT(rc);
	return rc;
};

void utils_get_create_options(option_array_t *options, luks_private_data_t *pdata)
{
	u_int i;

	LOG_ENTRY();

	strncpy(pdata->cipher, LUKS_CIPHER_DEFAULT, EVMS_NAME_SIZE);
	pdata->password[0] = 0;

/*	No need anymore
 *
	for (i = 0; i < options->count; i++)
	{
		if (options->option[i].is_number_based)
		{
			if (options->option[i].number == LUKS_CREATE_CIPHER_INDEX) strncpy(pdata->cipher, options->option[i].value.s, EVMS_NAME_SIZE);
			else if (options->option[i].number == LUKS_CREATE_PASSWORD_INDEX) strncpy(pdata->password, options->option[i].value.s, EVMS_NAME_SIZE);
		}
		else
		{
			if (strcmp(options->option[i].name, LUKS_CREATE_CIPHER_NAME) == 0) strncpy(pdata->cipher, options->option[i].value.s, EVMS_NAME_SIZE);
			else if (strcmp(options->option[i].name, LUKS_CREATE_PASSWORD_NAME) == 0) strncpy(pdata->password, options->option[i].value.s, EVMS_NAME_SIZE);
		}
	}
*/

	LOG_EXIT_VOID();
};

storage_object_t* utils_malloc_luks_object(void)
{
	int rc;
	storage_object_t *object = NULL;
	luks_private_data_t *pdata;

	LOG_ENTRY();

	rc = EngFncs->allocate_region(NULL, &object);
	if (rc == 0)
	{
		pdata = (luks_private_data_t*)calloc(1, sizeof(luks_private_data_t));
		if (pdata == NULL)
		{
			LOG_DEBUG("malloc luks private data failed\n");
			EngFncs->free_evms_object(object);
			object = NULL;
		}
		else
		{
			memset(pdata->password, 0, EVMS_NAME_SIZE + 1);
			object->plugin = my_plugin_record;
			object->private_data = pdata;
		}
	}
	else LOG_DEBUG("allocate evms object failed\n");

	LOG_EXIT_VOID();

	return object;
};

void utils_free_luks_object(storage_object_t *object)
{
	LOG_ENTRY();

	if (object)
	{
		luks_private_data_t *pdata = (luks_private_data_t*)object->private_data;
		if (pdata)
		{
			EngFncs->unregister_name(object->name);
			EngFncs->unregister_name(object->uuid);
			free(object->private_data);
			object->private_data = NULL;
		}
		EngFncs->free_region(object);
	}

	LOG_EXIT_VOID();
};

boolean utils_isa_luks(storage_object_t *object)
{
	if (object != NULL && object->plugin == my_plugin_record && object->private_data != NULL) return TRUE;
	return FALSE;
};

int utils_delete_object(storage_object_t *object, list_anchor_t child_objects, boolean destroy)
{
	storage_object_t *child;
	struct plugin_functions_s *fncs;

	LOG_ENTRY();

	REQUIRE(utils_isa_luks(object)==TRUE);
	REQUIRE(EngFncs->list_count(object->child_objects) == 1);

	child = EngFncs->first_thing(object->child_objects, NULL);

	if (destroy)
	{
		fncs = (struct plugin_functions_s *)child->plugin->functions.plugin;
		fncs->add_sectors_to_kill_list(child, 0, LUKS_METADATA_SECTOR_COUNT);
	}

	EngFncs->remove_thing(child->parent_objects, object);
	if (child_objects) EngFncs->insert_thing(child_objects, child, INSERT_BEFORE, NULL);

	utils_free_luks_object(object);

	LOG_EXIT_INT(0);
	return 0;
};

static int crypt_parse_name_and_mode(const char *s, char *cipher, int *key_nums, char *cipher_mode)
{
	if (sscanf(s, "%" MAX_CIPHER_LEN_STR "[^-]-%" MAX_CIPHER_LEN_STR "s", cipher, cipher_mode) == 2)
	{
		if (!strcmp(cipher_mode, "plain")) strncpy(cipher_mode, "cbc-plain", 10);
		if (key_nums)
		{
			char *tmp = strchr(cipher, ':');
			*key_nums = tmp ? atoi(++tmp) : 1;
			if (!*key_nums) return -EINVAL;
		}
		return 0;
	}

	if (sscanf(s, "%" MAX_CIPHER_LEN_STR "[^-]", cipher) == 1)
	{
		strncpy(cipher_mode, "cbc-plain", 10);
		if (key_nums) *key_nums = 1;
		return 0;
	}

	return -EINVAL;
};

static int build_crypt_tmpfile_and_write_to_device(const char *fname, char *cipher, char *password, char *uuid, storage_object_t *object)
{
	struct crypt_device *cd;
	struct crypt_params_luks1 params;
	int rc;
	char cipher_name[MAX_CIPHER_LEN + 1], cipher_mode[MAX_CIPHER_LEN + 1];
	FILE *f;
	int i;
	char buf[BUF_SIZE];

	LOG_ENTRY();

	REQUIRE(password != NULL);

	LOG_DEBUG("Create LUKS\n");
	if (cipher != NULL) LOG_DEBUG(" cipher: %s\n", cipher);
	if (uuid != NULL) LOG_DEBUG(" uuid: %s\n", uuid);

	rc = crypt_init(&cd, fname);
	if (rc < 0)
	{
		LOG_DEBUG("crypt_init() failed\n");
		LOG_EXIT_INT(rc);
		return rc;
	}

	LOG_DEBUG("Context is attached to block device %s\n", crypt_get_device_name(cd));

	params.hash = "sha1";
	params.data_alignment = 0;
	params.data_device = NULL;

	rc = crypt_parse_name_and_mode(cipher, cipher_name, NULL, cipher_mode);
	if (rc < 0)
	{
		LOG_DEBUG("No known cipher specification pattern detected\n");
		crypt_free(cd);
		LOG_EXIT_INT(rc);
		return rc;
	}

	if (uuid != NULL && uuid[0] == 0) uuid = NULL;

	// TODO: set variable key size
	rc = crypt_format(cd, CRYPT_LUKS1, cipher_name, cipher_mode, uuid, NULL, 256 / 8, &params);

	if (rc < 0)
	{
		LOG_DEBUG("crypt_format() failed on device %s\n", crypt_get_device_name(cd));
		crypt_free(cd);
		LOG_EXIT_INT(rc);
		return rc;
	}

	rc = crypt_keyslot_add_by_volume_key(cd, CRYPT_ANY_SLOT, NULL, 0, password, strlen(password));

	if (rc < 0)
	{
		LOG_DEBUG("Adding keyslot failed\n");
		crypt_free(cd);
		LOG_EXIT_INT(rc);
		return rc;
	}

	LOG_DEBUG("The keyslot is initialized\n");

	f = fopen(fname, "rb");
	if (f == NULL)
	{
		crypt_free(cd);
		LOG_DEBUG("cannot open '%s' file\n", fname);
		LOG_EXIT_INT(EBUSY);
		return EBUSY;
	}

	for (i = 0; i < 4096; i++)
	{
		if (fread(buf, 1, BUF_SIZE, f) == 0)
		{
			fclose(f);
			crypt_free(cd);
			LOG_DEBUG("error on read '%s' file\n", fname);
			LOG_EXIT_INT(EBUSY);
			return EBUSY;
		}
		if (WRITE(object, i, BUF_SECTOR_COUNT, buf) != 0)
		{
			fclose(f);
			crypt_free(cd);
			LOG_DEBUG("cannot write to object\n");
			LOG_EXIT_INT(EBUSY);
			return EBUSY;
		}
	}

	fclose(f);
	crypt_free(cd);

	LOG_EXIT_INT(0);
	return 0;
};

static int create_tmpfile(const char *fname)
{
	FILE *f;

	LOG_ENTRY();

	f = fopen(fname, "wb");
	if (f == NULL)
	{
		LOG_DEBUG("cannot create or open '%s' file\n", fname);
		LOG_EXIT_INT(1);
		return 1;
	}
	if (fseek(f, 4096 * EVMS_VSECTOR_SIZE - 1, SEEK_SET) == -1)
	{
		fclose(f);
		remove(fname);
		LOG_DEBUG("cannot resize '%s' file\n", fname);
		LOG_EXIT_INT(2);
		return 2;
	}
	if (fwrite("", 1, 1, f) == 0)
	{
		fclose(f);
		remove(fname);
		LOG_DEBUG("cannot resize '%s' file\n", fname);
		LOG_EXIT_INT(3);
		return 3;
	}
	fclose(f);

	LOG_EXIT_INT(0);
	return 0;
};

int utils_create_luks(storage_object_t *object, uint commit_phase)
{
	int rc = 0;
	storage_object_t *child;
	luks_private_data_t *pdata;

	LOG_ENTRY();

	REQUIRE(EngFncs->list_count(object->child_objects) == 1);

	child = EngFncs->first_thing(object->child_objects, NULL);
	pdata = (luks_private_data_t*)object->private_data;

	rc = create_tmpfile(TMP_FILE);
	if (rc == 0)
	{
		rc = build_crypt_tmpfile_and_write_to_device(TMP_FILE, pdata->cipher, pdata->password, object->uuid, child);
		if (remove(TMP_FILE) == -1) LOG_DEBUG("cannot delete '%s' file\n", TMP_FILE);
	}

	LOG_EXIT_INT(rc);
	return rc;
};

static int copy_header_to_tmpfile(const char *fname, storage_object_t *object)
{
	int i;
	char buf[BUF_SIZE];
	FILE *f;

	LOG_ENTRY();

	f = fopen(fname, "wb");
	if (f == NULL)
	{
		LOG_DEBUG("cannot open '%s' file\n", fname);
		LOG_EXIT_INT(EBUSY);
		return EBUSY;
	}

	for (i = 0; i < 4096; i++)
	{
		if (READ(object, i, BUF_SECTOR_COUNT, buf) != 0)
		{
			fclose(f);
			LOG_DEBUG("cannot read from object\n");
			LOG_EXIT_INT(EBUSY);
			return EBUSY;
		}
		if (fwrite(buf, 1, BUF_SIZE, f) == 0)
		{
			fclose(f);
			LOG_DEBUG("cannot write '%s' file\n", fname);
			LOG_EXIT_INT(ENOMEM);
			return ENOMEM;
		}
	}

	fclose(f);
	LOG_EXIT_INT(0);
	return 0;
};

static int check_password_and_build_key(const char *fname, const char *password, char *key)
{
	struct crypt_device *cd;
	int rc, i;
	size_t vk_size = EVMS_NAME_SIZE / 2;
	char vk[EVMS_NAME_SIZE / 2];

	LOG_ENTRY();

	REQUIRE(password != NULL);

	rc = crypt_init(&cd, fname);
	if (rc < 0)
	{
		LOG_DEBUG("crypt_init() failed\n");
		LOG_EXIT_INT(rc);
		return rc;
	}

	rc = crypt_load(cd, CRYPT_LUKS1, NULL);
	if (rc < 0)
	{
		LOG_DEBUG("crypt_load() failed on device %s.\n", crypt_get_device_name(cd));
		crypt_free(cd);
		LOG_EXIT_INT(rc);
		return rc;
	}

	rc = crypt_volume_key_get(cd, CRYPT_ANY_SLOT, vk, &vk_size, password, strlen(password));
	if (rc < 0)
	{
		LOG_DEBUG("crypt_volume_key_get() failed\n");
		crypt_free(cd);
		LOG_EXIT_INT(rc);
		return rc;
	}

	for (i = 0; i < vk_size; i++) sprintf(key + 2*i, "%.2x", (unsigned char)vk[i]);

	crypt_free(cd);

	LOG_EXIT_INT(0);
	return 0;
};

int utils_activate_luks(storage_object_t *object)
{
	storage_object_t *child;
	dm_target_crypt_t *dev;
	dm_target_t *trgt;
	luks_private_data_t *pdata;
	int rc;

	LOG_ENTRY();

	REQUIRE(EngFncs->list_count(object->child_objects) == 1);

	child = EngFncs->first_thing(object->child_objects, NULL);
	pdata = (luks_private_data_t*)object->private_data;

	trgt = EngFncs->dm_allocate_target(DM_TARGET_CRYPT, object->start, object->size, 0, 0);
	if (!trgt)
	{
		LOG_EXIT_INT(ENOMEM);
		return ENOMEM;
	}

	dev = trgt->data.crypt;
	dev->device.major = child->dev_major;
	dev->device.minor = child->dev_minor;
	dev->device.start = 4096;	// TODO: get from pdata/metadata
	dev->iv_offset = 0;
	strncpy(dev->cypher, pdata->cipher, EVMS_NAME_SIZE);

	rc = create_tmpfile(TMP_FILE);
	if (rc == 0)
	{
		rc = copy_header_to_tmpfile(TMP_FILE, child);
		if (rc == 0) rc = check_password_and_build_key(TMP_FILE, pdata->password, dev->key);

		if (remove(TMP_FILE) == -1) LOG_DEBUG("cannot delete '%s' file\n", TMP_FILE);

		if (rc != 0)
		{
			EngFncs->dm_deallocate_targets(trgt);
			LOG_EXIT_INT(rc);
			return rc;
		}
	}

	rc = EngFncs->dm_activate(object, trgt);
	EngFncs->dm_deallocate_targets(trgt);

	LOG_EXIT_INT(rc);
	return rc;
};

int utils_deactivate_luks(storage_object_t *object)
{
	int rc;

	LOG_ENTRY();

	rc = EngFncs->dm_deactivate(object);

	LOG_EXIT_INT(rc);
	return rc;
};

int utils_read_and_check_metadata(storage_object_t *object, luks_metadata_t *metadata)
{
	char *buffer;
	int rc = 0;

	LOG_ENTRY();

	buffer = EngFncs->engine_alloc(LUKS_METADATA_SECTOR_COUNT * EVMS_VSECTOR_SIZE);
	if (!buffer)
	{
		LOG_EXIT_INT(ENOMEM);
		return ENOMEM;
	}

	rc = READ(object, 0, LUKS_METADATA_SECTOR_COUNT, buffer);
	if (rc)
	{
		EngFncs->engine_free(buffer);
		LOG_EXIT_INT(rc);
		return rc;
	}

	memcpy(metadata, buffer, sizeof(*metadata));
	EngFncs->engine_free(buffer);

	if (memcmp(metadata->magic, LUKS_MAGIC, LUKS_MAGIC_SIZE) == 0)
	{
		LOG_EXIT_INT(0);
		return 0;
	}

	LOG_EXIT_INT(ENOENT);
	return ENOENT;
};

void utils_fill_pdata_from_metadata(luks_private_data_t *pdata, luks_metadata_t *metadata)
{
	char cipherName[LUKS_CIPHERNAME_L + 1];
	char cipherMode[LUKS_CIPHERMODE_L + 1];
	char uuid[LUKS_UUID_STRING_L + 1];

	LOG_ENTRY();

	strncpy(cipherName, metadata->cipherName, LUKS_CIPHERNAME_L);
	strncpy(cipherMode, metadata->cipherMode, LUKS_CIPHERMODE_L);
	strncpy(uuid, metadata->uuid, LUKS_UUID_STRING_L);

	snprintf(pdata->cipher, EVMS_NAME_SIZE, "%s-%s", cipherName, cipherMode);
	// TODO: write UUID to pdata and show it in details dialog

	LOG_EXIT_VOID();
	return;
};


void utils_get_devmap_info(storage_object_t *object)
{
	luks_private_data_t *pdata = (luks_private_data_t*)object->private_data;
	storage_object_t *child = EngFncs->first_thing(object->child_objects, NULL);
	dm_target_t *trgts;
	int rc;

	LOG_ENTRY();

	EngFncs->dm_update_status(object);
	object->flags &= ~SOFLAG_NEEDS_ACTIVATE;

	LOG_EXIT_VOID();
};
