/*
 *   (C) Copyright IBM Corp. 2001, 2003
 *   (C) Copyright ALT Linux Ltd. 2011
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Module: mdregmgr
 * File: raid10_mgr.h
 */

#ifndef _RAID10_MGR_H_
#define _RAID10_MGR_H_ 1


struct strip_zone
{
	sector_count_t     zone_offset;		/* Zone offset in md_dev */
	sector_count_t     dev_offset;		/* Zone offset in real dev */
	sector_count_t     size;       		/* Zone size */
	int                nb_dev;		/* # of devices attached to the zone */
	md_member_t      **dev;			/* Devices attached to the zone */
};

struct raid10_hash
{
	struct strip_zone * zone0;
	struct strip_zone * zone1;
};

typedef struct raid10_private_data
{
	struct raid10_hash * hash_table;
	struct strip_zone * strip_zone;
	struct strip_zone * smallest_zone;
	int nr_strip_zones;
	int nr_zones;
	u_int32_t chunksize;	/* in sectors */
} raid10_conf_t;


#define mdvol_to_conf(mdvol) ((raid10_conf_t *) mdvol->private_data)

/* Create options */
#define RAID10_CREATE_OPT_COUNT			3

#define RAID10_CREATE_OPT_SB1_INDEX		0
#define RAID10_CREATE_OPT_SB1_NAME		"ver1_superblock"

#define RAID10_CREATE_OPT_CHUNK_SIZE_INDEX	1
#define RAID10_CREATE_OPT_CHUNK_SIZE_NAME	"chunksize"

#define RAID10_CREATE_OPT_LAYOUT_NEAR_INDEX	2
#define RAID10_CREATE_OPT_LAYOUT_NEAR_NAME	"layout_near"


/* Expand/Shrink options */
#define RAID10_EXPAND_OPTION_COUNT		0
#define RAID10_SHRINK_OPTION_COUNT		0

// Plug-in functions : none
#define MD_RAID10_FUNCTION_COUNT		0

typedef struct raid10_delay_kill_sector_s
{
	storage_object_t *region;
	lsn_t lsn;
	sector_count_t count;
} raid10_delay_kill_sector_t;

#define RAID10_DEFAULT_LAYOUT_NEAR		2

#endif

